BODY = "body"
FIRST_SEEN = "firstSeen"
HOST = "host"  # refers to user's local hostname NOT the URL host
SCRAPE_STATUS = "scrapeStatus"
TABS = "safari_tabs"
TAB_INDEX = "tabIndex"
TITLE = "title"
URL = "url"
WINDOW_ID = "windowId"

TURSO_SAFARI = "turso_safari"
TURSO_URL = "turso_url"
TURSO_AUTH_TOKEN = "turso_auth_token"  # noqa: S105

NOT_SCRAPED = -1
EXTRACT_FAILED = -2
UNICODE_FAILED = -3

SEP = "150r2M72e7D6Lb7Z9u1g4BSQBonv6U21W1fmX8B1TXR8XXqB2wTgQzqwoB06144d"
